//+------------------------------------------------------------------+
//|                                              ZigZag_Validity.mq4 |
//| version 0.0 pre-beta                                             |
//|                                                                  |
//| Based on the original ZigZag validity indicator for MetaStock    |
//| created by Spyros Raftopoulos                                    |
//| http://trader.online.pl/MSZ/e-w-ZigZag_Validity.html             |
//| It validates the LAST LEG of the Zigzag indicator.               |
//| 1=valid (not revisable), 0=invalid (revisable).                  |
//|                                                                  |
//|                                                 MT4 code by NiX  |
//|                                                                  |
//|                          Donate by PayPal to: artur@lunar.com.pl |
//+------------------------------------------------------------------+

#property copyright "Copyright  2007, NiX"

#property indicator_separate_window
#property indicator_minimum -0.2
#property indicator_maximum 1.2
#property indicator_buffers 2
#property indicator_color1 Yellow

extern double percent = 0.5;

//---- buffers
double ZigZagValBuffer[];
double ZigZagBuffer[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{

   if(percent < 0.2)
      percent = 0.2;
   else if(percent > 100)
      percent = 100;

   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,ZigZagValBuffer);
   SetIndexStyle(1,DRAW_NONE);
   SetIndexBuffer(1,ZigZagBuffer);
   IndicatorShortName("ZigZag Validity ("+DoubleToStr(percent,2)+"%) = ");

   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+

int start()
{
   double zzpUp = 0, zzpDn = 0;
   int counted_bars=IndicatorCounted();
   if(counted_bars<0) 
      return(-1);

   int limit=Bars-counted_bars-1;

   for(int shift=0; shift<limit; shift++) 
   { 
      zzpDn = iCustom(Symbol(),Period(),"ZZP",0,percent,false,false,1,shift);     
      zzpUp = iCustom(Symbol(),Period(),"ZZP",0,percent,false,false,0,shift);      

     if(zzpUp != EMPTY_VALUE) 
         ZigZagBuffer[shift] = zzpUp;
     else
         ZigZagBuffer[shift] = zzpDn;
   }
   
   for(shift=0; shift<limit; shift++)
     ZigZagValBuffer[shift] = calcZigZagValidity(shift); 
   
   return(0);
}

//+------------------------------------------------------------------+

double last = 0;
int    SD   = 0;
double pc   = 0;
int    res  = 0;

double calcZigZagValidity(int shift)
{
  if(((ZigZagBuffer[shift] > ZigZagBuffer[shift+1]) && 
      (ZigZagBuffer[shift+1] < ZigZagBuffer[shift+2])) ||
      ((ZigZagBuffer[shift] < ZigZagBuffer[shift+1]) && 
      (ZigZagBuffer[shift+1] > ZigZagBuffer[shift+2])))
         last = ZigZagBuffer[shift+1];

   if(last != 0)
      pc = MathAbs((iClose(Symbol(),Period(),shift)-last) * 100 / last);   
   else
      pc = 0;
   
   //
   // The original MetaStock code introduces the SD variable
   // it is calculated the following way:
   // SD:=(z>Ref(z,-1) AND Ref(z,-1)>Ref(z,-2)) OR (z<Ref(z,-1) AND Ref(z,-1)<Ref(z,-2)); 
   //
   // My understanding of this code is that it verified weather the ZigZag indicator
   // is showing higher highs or lower lows.
   //
   // I need to work on this... Don't know if the result is boolean or some other value...
   // 
       
   if(((ZigZagBuffer[shift] > ZigZagBuffer[shift+1]) && (ZigZagBuffer[shift+1] > ZigZagBuffer[shift+2])) ||
      ((ZigZagBuffer[shift] < ZigZagBuffer[shift+1]) && (ZigZagBuffer[shift+1] < ZigZagBuffer[shift+2])))
         SD = 1;
   
   if(pc >= percent)
      res = 1;
   else
      res = 0;

   //
   // the rest of this function is a termporary patch since I still don't fully understand the
   // following MetaStock code (mainly the Alert function).
   //
   // If(Alert(res,2) AND SD,1,res)
   //
   // Still needs some work
   //
   
   return(res);   
}

//+------------------------------------------------------------------+
//| The original MetaStock code                                      |
//+------------------------------------------------------------------+
/*

      {*********** Start of code *************}

      {ZigZag validity by Spyros Raftopoulos
      It validates the LAST LEG of the Zigzag indicator.
      1=valid (not revisable), 0=invalid (revisable).
      Do NOT use this indicator in systems.}

      perc:=Input("Percent",0.2,100,10);

      Z:=Zig(CLOSE,perc,%);
      last:=ValueWhen(1,( Z > Ref(Z,-1) AND Ref(Z,-1) < Ref(Z,-2) )
      OR
      ( Z < Ref(Z,-1) AND Ref(Z,-1) > Ref(Z,-2) ),
      Ref(Z,-1));

      pc:=(CLOSE-last) * 100 / last;
      pc:= Abs(pc);

      SD:=(z>Ref(z,-1) AND Ref(z,-1)>Ref(z,-2)) OR (z<Ref(z,-1) AND Ref(z,-
      1)<Ref(z,-2));

      res:=If(pc>=perc ,1,0);
      If(Alert(res,2) AND SD,1,res)

      {*********** End of code *************}

*/

